function nan_gplotmatrix(x,y,g,colorlist,symbol,markerSize,doLegend,dispopt,xnam,ynam)
//  Scatter plot matrix with grouping variable.
//  Calling Sequence
//  nan_gplotmatrix(X,Y,G)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG,DISPOPT)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG,DISPOPT,XNAM,YNAM)
//  nan_gplotmatrix(X,[],G,...)
//  Parameters
//  X: P by M matrix
//  Y: if Y is a P by N Matrix, N by M scatter plots will be created. If y is empty X vs X will be plotted
//  G: grouping  variable that determines the marker and color assigned to each point in   each matrix
//  CLR : string of color specifications. Default is 'bgrcmyk'
//  SYM : string of marker specifications. Default is '.'
//  SIZ : marker size to use  for all plots 
//  DOLEG : 'on' legends are plotted. 'off' means no legens will be plotted at all
//  DISPOPT : lets you control how to  fill the diagonals in a plot of X vs. X. Set DISPOPT to 'none' to  leave them blank, 'hist' (default) to plot histograms, or 'variable' to   write the variable names.
//  XNAM : names of the X variables, must be a   character array or cell array of strings of the appropriate dimension.
//  YNAM : names of the Y variables, must be a   character array or cell array of strings of the appropriate dimension.
//  H: handle of the plotted points
//  AX: handle of the subaxes
//  BigAx : handle to the figure
//  Description
//   nan_gplotmatrix creates a matrix of scatter plots of the columns of
//   X against the columns of Y, grouped by G.  If Y is [], scatter plots of X vs X will be plotted.
//   
//   If the number of columns of X is M and the number of columns of Y is N, then N by M scatter plots will be created.
//
//   Examples
//     loadmatfile(nan_getpath()+"/demos/data/iris.mat");
//      varNames = {'Sepal Length','Sepal Width','Petal Length','Petal Width'};
//      nan_gplotmatrix(meas,[],species,'bgrcm',[],[],'on','hist',varNames');
//
//   See also
//   nan_grpstats
//   nan_gscatter
//   nan_plotmatrix
//   Authors
//   H. Nahrstaedt


[nargout,nargin]=argn(0);
if nargin<1
    error('Requires one Parameter');
end;

nin = nargin;

if (nin < 2), y = []; end
if isempty(y)
  rows = size(x,2); cols = rows;
  y = x;
  XvsX = %t;
else 
  rows = size(y,2); cols = size(x,2);
  XvsX = %f;
end
if (nin > 2) & ~isempty(g)
   [g,gn] = nan_mgrp2idx(g,size(x,1),',');
   ng = max(g);
else
   g = [];
   gn = [];
   ng = 1;
end

if (nin < 4) | isempty(colorlist), colorlist = 'bgrcmyk'; end
if (nin < 5) | isempty(symbol), symbol = '.'; end
if (nin < 6), markerSize = []; end
if (nin < 7) | isempty(doLegend), doLegend = 'on'; end
if (nin < 8) | isempty(dispopt), dispopt = 'h'; end
if (nin < 9) | isempty(xnam)
   xnam = {};
else
   if type(xnam)==10 & (size(xnam,1)==cols)
       xnam = cellstr(xnam);
   elseif typeof(xnam)=="ce" & (length(xnam)==cols)
      // ok
   else
      error('xnam must be a string vector or cell!');
   end
end
if (XvsX)
   ynam = xnam;
elseif (nin < 10) | isempty(ynam)
   ynam = {};
else
   if type(ynam)==10 & (size(ynam,1)==rows)
       ynam = cellstr(ynam);
   elseif typeof(ynam)=="ce" & (length(ynam)==rows)
      // ok
   else
      error('ynam must be a string vector or cell!');
   end
end

doLegend = (doLegend== 'on') & (~XvsX | (rows>1)) & ~isempty(gn);
have_hist = XvsX & (part(dispopt,1)=='h');
donames = (XvsX & (part(dispopt,1)=='v'));

if isempty(rows) | isempty(cols),
   if nargout>0, h = []; ax = []; BigAx = []; end
   return
end

if length(size(x))>2 | length(size(y))>2
   error('X and Y must be a vector or a matrix');
end
if size(x,1)~=size(y,1)
  error( 'X and Y must have the same length.');
end
if (~isempty(g)) & (length(g) ~= size(x,1)),
  error( 'The lenght of g is wrong!');
end

scf();
BigAx = gca();


if (isempty(markerSize))
   markerSize = repmat(3, size(symbol));
   if ~isempty(strindex(symbol,'.')),

      markerSize(symbol=='.') = max(0,min(15,round(1*min(get(BigAx.parent,'figure_size'))/size(x,1)/max(rows,cols))));
   end
end
if donames
  offset=30;
else
  offset=50;
end


  xsize = 0.9 / cols;
  ysize = 0.9 / rows;
  xoff = 0.05;
  yoff = 0.05;
  border = [0.130, 0.110, 0.225, 0.185] .* [xsize, ysize, xsize, ysize];
  border(3:4) = - border(3:4) - border(1:2);

[m,n] = size(y); 

 xlim = zeros(rows, cols, 2);
 ylim = zeros(rows, cols, 2);

ax=list();
for i=rows:-1:1,
   for j=cols:-1:1,

      ax(rows*(i-1)+j) = newaxes();
      ax(rows*(i-1)+j).axes_bounds=[xsize * (j - 1) + xoff, ysize * ( i -1) + yoff, xsize, ysize]';
      ax(rows*(i-1)+j).tight_limits="on";
      if ((i==j) & XvsX)
         if (have_hist)
            [nn,xx] = nan_hist(y(:,i));
            bar(xx,nn,1.0);
              ax(rows*(i-1)+j).auto_ticks=["on","on","on"];
            //axis tight;
            
            xlim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,1);
            ylim(i,j,:) = [%inf, -%inf];
           end
      end
end;
end;
for i=rows:-1:1,
   for j=cols:-1:1,

      sca(ax(rows*(i-1)+j));
     
      if ((i==j) & XvsX)
//          if (have_hist)
//             [nn,xx] = nan_hist(y(:,i));
//             bar(xx,nn,1.0);
//               ax(rows*(i-1)+j).auto_ticks=["on","on","on"];
//             //axis tight;
//             ax(rows*(i-1)+j).tight_limits="on";
//             xlim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,1);
//             ylim(i,j,:) = [%inf, -%inf];
//            end
      else
           xj=x(:,j);yi=y(:,i);


	nm = length(symbol);
	ns = length(markerSize);
	nc = length(colorlist);

	for jj=1:ng
	  if sum((g == jj))==0
		    plot(%nan,%nan  ,[part(colorlist,(1+pmodulo(jj-1,nc)))+ part(symbol,(1+pmodulo(jj-1,nm)))]);
	  else
		plot(xj((g == jj),:), yi((g == jj),:),[part(colorlist,(1+pmodulo(jj-1,nc)))+ part(symbol,(1+pmodulo(jj-1,nm)))]);
	      
	      if ~isempty(markerSize)	
                        hhij=gce();
			hhij.children(1).mark_size=markerSize(1+pmodulo(jj-1,ns));
		end
	  end
	end

         ylim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,2);
         xlim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,1);


      end
   end
end



 xlimmin = min(xlim(:,:,1),'r'); xlimmax = max(xlim(:,:,2),'r');
 ylimmin = min(ylim(:,:,1),'c'); ylimmax = max(ylim(:,:,2),'c');

inset = .05;


 for i=1:rows,
  for j=1:cols

      ax(rows*(i-1)+j).data_bounds(:,1)=[xlimmin(1,j) xlimmax(1,j)]';
      dx = diff(ax(rows*(i-1)+j).data_bounds(:,1))*inset;

     ax(rows*(i-1)+j).data_bounds(:,1)=[xlimmin(1,j)-dx xlimmax(1,j)+dx]';

      if (i==j & have_hist )
//        ax(rows*(i-1)+j).data_bounds(:,2)=[0 ylimmax(i,1)]';
//        dy = diff(ax(rows*(i-1)+j).data_bounds(:,2))*inset;
//        ax(rows*(i-1)+j).data_bounds(:,2)=[0 ylimmax(i,1)+dy]';
      else
        ax(rows*(i-1)+j).data_bounds(:,2)=[ylimmin(i,1) ylimmax(i,1)]';
        dy = diff(ax(rows*(i-1)+j).data_bounds(:,2))*inset;
        ax(rows*(i-1)+j).data_bounds(:,2)=[ylimmin(i,1)-dy ylimmax(i,1)+dy]';
     end;
  end;
 end


if (donames & ~isempty(xnam))
   for j=1:cols
       sca(ax(rows*(j-1)+j));
      xstring((  xlimmin(1,j)), (ylimmin(j,1)+ylimmax(j,1))/2,    xnam(j).entries);       
   end
else
   if ~isempty(xnam)
      for j=1:cols, xlabel(ax(rows*(rows-1)+j),xnam(j).entries); end
   end
   if ~isempty(ynam)
      for i=1:rows, ylabel(ax(rows*(i-1)+1),ynam(i).entries); end
   end
end

//  for i=1:rows-1,
//   for j=2:cols
//   ax(rows*(i-1)+j).x_ticks.labels=[''];
//   ax(rows*(i-1)+j).y_ticks.labels=[''];
//    ax(rows*(i-1)+j).margins=[0.01 0.01 0.01 0.01];
// end
//   
// end
// for j=2:cols
//  ax(rows*(rows-1)+j).y_ticks.labels=[''];
//  ax(rows*(rows-1)+j).margins=[0.01 0.01 0.01 0.01];
// end;
//  for i=1:rows-1,
//  ax(rows*(i-1)+1).x_ticks.labels=[''];
// ax(rows*(i-1)+1).margins=[0.01 0.01 0.01 0.01];
// end;
// ax(rows*(rows-1)+1).margins=[0.01 0.01 0.01 0.01];
sca(BigAx);
if (doLegend)
   legend(ax(rows*(1-1)+cols),gn,-1);
end





endfunction